package handlers

import (
	"strconv"
	"ttime/internal/types"

	"github.com/gofiber/fiber/v2"
	"github.com/golang-jwt/jwt/v5"
)

func (gs *GState) SubmitWeeklyReport(c *fiber.Ctx) error {
	// Extract the necessary parameters from the token
	user := c.Locals("user").(*jwt.Token)
	claims := user.Claims.(jwt.MapClaims)
	username := claims["name"].(string)

	report := new(types.NewWeeklyReport)
	if err := c.BodyParser(report); err != nil {
		return c.Status(400).SendString(err.Error())
	}

	// Make sure all the fields of the report are valid
	if report.Week < 1 || report.Week > 52 {
		return c.Status(400).SendString("Invalid week number")
	}
	if report.DevelopmentTime < 0 || report.MeetingTime < 0 || report.AdminTime < 0 || report.OwnWorkTime < 0 || report.StudyTime < 0 || report.TestingTime < 0 {
		return c.Status(400).SendString("Invalid time report")
	}

	if err := gs.Db.AddWeeklyReport(report.ProjectName, username, report.Week, report.DevelopmentTime, report.MeetingTime, report.AdminTime, report.OwnWorkTime, report.StudyTime, report.TestingTime); err != nil {
		return c.Status(500).SendString(err.Error())
	}

	return c.Status(200).SendString("Time report added")
}

// Handler for retrieving weekly report
func (gs *GState) GetWeeklyReport(c *fiber.Ctx) error {
	// Extract the necessary parameters from the request
	println("GetWeeklyReport")
	user := c.Locals("user").(*jwt.Token)
	claims := user.Claims.(jwt.MapClaims)
	username := claims["name"].(string)

	// Extract project name and week from query parameters
	projectName := c.Query("projectName")
	println(projectName)
	week := c.Query("week")
	println(week)

	// Convert week to integer
	weekInt, err := strconv.Atoi(week)
	if err != nil {
		return c.Status(400).SendString("Invalid week number")
	}

	// Call the database function to get the weekly report
	report, err := gs.Db.GetWeeklyReport(username, projectName, weekInt)
	if err != nil {
		return c.Status(500).SendString(err.Error())
	}

	// Return the retrieved weekly report
	return c.JSON(report)
}

type ReportId struct {
	ReportId int
}

func (gs *GState) SignReport(c *fiber.Ctx) error {
	println("Signing report...")
	// Extract the necessary parameters from the token
	user := c.Locals("user").(*jwt.Token)
	claims := user.Claims.(jwt.MapClaims)
	projectManagerUsername := claims["name"].(string)

	// Extract report ID from the request query parameters
	// reportID := c.Query("reportId")
	rid := new(ReportId)
	if err := c.BodyParser(rid); err != nil {
		return err
	}
	println("Signing report for: ", rid.ReportId)
	// reportIDInt, err := strconv.Atoi(rid.ReportId)
	// println("Signing report for: ", rid.ReportId)
	// if err != nil {
	// 	return c.Status(400).SendString("Invalid report ID")
	// }

	// Get the project manager's ID
	projectManagerID, err := gs.Db.GetUserId(projectManagerUsername)
	if err != nil {
		return c.Status(500).SendString("Failed to get project manager ID")
	}
	println("blabla", projectManagerID)

	// Call the database function to sign the weekly report
	err = gs.Db.SignWeeklyReport(rid.ReportId, projectManagerID)
	if err != nil {
		return c.Status(500).SendString(err.Error())
	}

	return c.Status(200).SendString("Weekly report signed successfully")
}
