import { useState } from "react";
import { api } from "../API/API";
import { useNavigate } from "react-router-dom";
import Button from "./Button";
import { NewWeeklyReport } from "../Types/goTypes";

export default function NewTimeReport(): JSX.Element {
  const [projectName, setProjectName] = useState<string>("projectName"); // TODO: Get from backend
  const [week, setWeek] = useState<number>(NaN);
  const [development, setDevelopment] = useState<number>(NaN);
  const [meeting, setMeeting] = useState<number>(NaN);
  const [administration, setAdministration] = useState<number>(NaN);
  const [ownwork, setOwnWork] = useState<number>(NaN);
  const [studies, setStudies] = useState<number>(NaN);
  const [testing, setTesting] = useState<number>(NaN);

  const handleNewTimeReport = async (): Promise<void> => {
    const newTimeReport: NewWeeklyReport = {
      projectName,
      week,
      developmentTime: development,
      meetingTime: meeting,
      adminTime: administration,
      ownWorkTime: ownwork,
      studyTime: studies,
      testingTime: testing,
    };
    await Promise.resolve();
    await api.submitWeeklyReport(newTimeReport, "token");
  };

  const navigate = useNavigate();

  setProjectName("Something Reasonable"); // This should obviously not be used here

  return (
    <>
      <div className="border-4 border-black bg-white flex flex-col justify-start min-h-[65vh] h-fit w-[50vw] rounded-3xl overflow-scroll space-y-[2vh] p-[30px] items-center">
        <form
          onSubmit={(e) => {
            if (!week) {
              alert("Please enter a week number");
              e.preventDefault();
              return;
            }
            e.preventDefault();
            void handleNewTimeReport();
            navigate("/project");
          }}
        >
          <div className="flex flex-col items-center">
            <input
              className="w-fill h-[5vh] font-sans text-[3vh] pl-[1vw] rounded-full text-center pt-[1vh] pb-[1vh] border-2 border-black"
              type="week"
              placeholder="Week"
              onChange={(e) => {
                const weekNumber = parseInt(e.target.value.split("-W")[1]);
                setWeek(weekNumber);
              }}
              onKeyDown={(event) => {
                event.preventDefault();
              }}
              onPaste={(event) => {
                event.preventDefault();
              }}
            />
            <table className="w-full text-center divide-y divide-x divide-white text-[30px]">
              <thead>
                <tr>
                  <th className="w-1/2 py-2 border-b-2 border-black">
                    Activity
                  </th>
                  <th className="w-1/2 py-2 border-b-2 border-black">
                    Total Time (min)
                  </th>
                </tr>
              </thead>
              <tbody className="divide-y divide-black">
                <tr className="h-[10vh]">
                  <td>Development</td>
                  <td>
                    <input
                      type="number"
                      min="0"
                      className="border-2 border-black rounded-md text-center w-1/2"
                      value={development}
                      onChange={(e) => {
                        setDevelopment(parseInt(e.target.value));
                      }}
                      onKeyDown={(event) => {
                        const keyValue = event.key;
                        if (!/\d/.test(keyValue) && keyValue !== "Backspace")
                          event.preventDefault();
                      }}
                    />
                  </td>
                </tr>
                <tr className="h-[10vh]">
                  <td>Meeting</td>
                  <td>
                    <input
                      type="number"
                      min="0"
                      className="border-2 border-black rounded-md text-center w-1/2"
                      value={meeting}
                      onChange={(e) => {
                        setMeeting(parseInt(e.target.value));
                      }}
                      onKeyDown={(event) => {
                        const keyValue = event.key;
                        if (!/\d/.test(keyValue) && keyValue !== "Backspace")
                          event.preventDefault();
                      }}
                    />
                  </td>
                </tr>
                <tr className="h-[10vh]">
                  <td>Administration</td>
                  <td>
                    <input
                      type="number"
                      min="0"
                      className="border-2 border-black rounded-md text-center w-1/2"
                      value={administration}
                      onChange={(e) => {
                        setAdministration(parseInt(e.target.value));
                      }}
                      onKeyDown={(event) => {
                        const keyValue = event.key;
                        if (!/\d/.test(keyValue) && keyValue !== "Backspace")
                          event.preventDefault();
                      }}
                    />
                  </td>
                </tr>
                <tr className="h-[10vh]">
                  <td>Own Work</td>
                  <td>
                    <input
                      type="number"
                      min="0"
                      className="border-2 border-black rounded-md text-center w-1/2"
                      value={ownwork}
                      onChange={(e) => {
                        setOwnWork(parseInt(e.target.value));
                      }}
                      onKeyDown={(event) => {
                        const keyValue = event.key;
                        if (!/\d/.test(keyValue) && keyValue !== "Backspace")
                          event.preventDefault();
                      }}
                    />
                  </td>
                </tr>
                <tr className="h-[10vh]">
                  <td>Studies</td>
                  <td>
                    <input
                      type="number"
                      min="0"
                      className="border-2 border-black rounded-md text-center w-1/2"
                      value={studies}
                      onChange={(e) => {
                        setStudies(parseInt(e.target.value));
                      }}
                      onKeyDown={(event) => {
                        const keyValue = event.key;
                        if (!/\d/.test(keyValue) && keyValue !== "Backspace")
                          event.preventDefault();
                      }}
                    />
                  </td>
                </tr>
                <tr className="h-[10vh]">
                  <td>Testing</td>
                  <td>
                    <input
                      type="number"
                      min="0"
                      className="border-2 border-black rounded-md text-center w-1/2"
                      value={testing}
                      onChange={(e) => {
                        setTesting(parseInt(e.target.value));
                      }}
                      onKeyDown={(event) => {
                        const keyValue = event.key;
                        if (!/\d/.test(keyValue) && keyValue !== "Backspace")
                          event.preventDefault();
                      }}
                    />
                  </td>
                </tr>
              </tbody>
            </table>
            <Button
              text="Submit"
              onClick={(): void => {
                return;
              }}
              type="submit"
            />
          </div>
        </form>
      </div>
    </>
  );
}
